import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOperator, Like, Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private moduloService: ModuloService,
  ) {}

  async create(
    id_tipo_carrito: number,
    id_modulo: number,
    marca: string,
    modelo: string,
  ) {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findTipoCarritoById(
        id_tipo_carrito,
      );

    return this.repository
      .count({ modulo, tipoCarrito })
      .then((carritos) =>
        this.repository.save(
          this.repository.create({
            carrito: `C${carritos >= 10 ? '' : '0'}${carritos + 1}`,
            marca,
            modelo,
            modulo,
            tipoCarrito,
          }),
        ),
      )
      .then((_) => ({
        message: 'Se creó correctamente el carrito en el módulo.',
      }));
  }

  async existeCarrito(id_modulo: number | Modulo, carrito: string) {
    const modulo =
      typeof id_modulo === 'number'
        ? await this.moduloService.findById(id_modulo)
        : id_modulo;

    return this.repository
      .findOne({ modulo, carrito })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException(
            'Ya existe un carrito con este nombre, intenta con otro.',
          );
      });
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este carrito.');
      return carrito;
    });
  }

  async findByIdModulo(filtros: {
    pagina: string;
    id_modulo: string;
    activo?: string;
    carrito?: string;
    id_tipo_carrito?: string;
  }) {
    const busqueda: {
      modulo: Modulo;
      activo?: boolean;
      carrito?: FindOperator<string>;
      tipoCarrito?: TipoCarrito;
    } = {
      modulo: await this.moduloService.findById(parseInt(filtros.id_modulo)),
    };
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;

    if (filtros.activo === 'true') busqueda.activo = true;
    if (filtros.activo === 'false') busqueda.activo = false;
    if (filtros.carrito) busqueda.carrito = Like(`%${filtros.carrito}%`);
    if (tipoCarrito) busqueda.tipoCarrito = tipoCarrito;
    return this.repository.findAndCount({
      where: busqueda,
      skip: (parseInt(filtros.pagina) - 1) * 25,
      take: 25,
    });
  }

  async findCarrito(
    id_modulo: number | Modulo,
    id_tipo_carrito: number | TipoCarrito,
    carrito: string,
    validarNoExiste = true,
  ) {
    const modulo =
      typeof id_modulo === 'number'
        ? await this.moduloService.findById(id_modulo)
        : id_modulo;
    const tipoCarrito =
      typeof id_tipo_carrito === 'number'
        ? await this.institucionTipoCarritoService.findTipoCarritoById(
            id_tipo_carrito,
          )
        : id_tipo_carrito;

    return this.repository
      .findOne({ modulo, carrito, tipoCarrito })
      .then((carrito) => {
        if (validarNoExiste && !carrito)
          throw new ConflictException('No existe este carrito.');
        return carrito;
      });
  }

  async update(attrs: {
    id_carrito: number;
    activo?: boolean;
    carrito?: string;
    id_modulo?: number;
    id_tipo_carrito?: number;
    marca?: string;
    modelo?: string;
  }) {
    return this.findById(attrs.id_carrito)
      .then(async (carrito) => {
        const modulo = attrs.id_modulo
          ? await this.moduloService.findById(attrs.id_modulo)
          : null;
        const tipoCarrito = attrs.id_modulo
          ? await this.institucionTipoCarritoService.findTipoCarritoById(
              attrs.id_tipo_carrito,
            )
          : null;

        if (attrs.carrito)
          await this.existeCarrito(carrito.modulo, attrs.carrito);
        delete attrs.id_tipo_carrito;
        delete attrs.id_modulo;
        Object.assign(carrito, attrs);
        if (modulo) carrito.modulo = modulo;
        if (tipoCarrito) carrito.tipoCarrito = tipoCarrito;
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se actualizó correctamente la información del carrito.',
      }));
  }
}
