import {
  ConflictException,
  NotFoundException,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { LessThanOrEqual, MoreThanOrEqual, Repository } from 'typeorm';
import { HoraExcepcion } from './entity/hora-excepcion.entity';
import { InstitucionDiaService } from '../institucion-dia/institucion-dia.service';
import { InstitucionDia } from 'src/institucion-dia/entity/institucion-dia.entity';

@Injectable()
export class HoraExcepcionService {
  constructor(
    @InjectRepository(HoraExcepcion)
    private repository: Repository<HoraExcepcion>,
    private institucionDiaService: InstitucionDiaService,
  ) {}

  async create(
    id_institucion_dia: number,
    hora_inicio: string,
    hora_fin: string,
  ) {
    const institucionDia = await this.institucionDiaService.findById(
      id_institucion_dia,
    );

    if (hora_inicio > hora_fin)
      throw new ConflictException(
        'La hora inicio no puede ser más grande que la hora fin.',
      );
    return this.validarHoras(institucionDia, hora_inicio, hora_fin)
      .then(() =>
        this.repository.save(
          this.repository.create({ institucionDia, hora_inicio, hora_fin }),
        ),
      )
      .then((_) => ({ message: 'Se creó correctamente la hora excepción.' }));
  }

  delete(id_hora_excepcion: number) {
    return this.findById(id_hora_excepcion).then((horaExcepcion) =>
      this.repository.delete(horaExcepcion),
    );
  }

  findAllByIdInstitucionDia(id_institucion_dia: number) {
    return this.institucionDiaService
      .findById(id_institucion_dia)
      .then((institucionDia) => this.repository.find({ institucionDia }));
  }

  findById(id_hora_excepcion: number) {
    return this.repository
      .findOne({ id_hora_excepcion })
      .then((horaExcepcion) => {
        if (!horaExcepcion)
          throw new NotFoundException('No existe la hora excepción.');
        return horaExcepcion;
      });
  }

  update(attrs: Partial<HoraExcepcion>) {
    if (!attrs.hora_inicio && !attrs.hora_fin)
      throw new ConflictException('No se mando nada para actualizar.');
    return this.findById(attrs.id_hora_excepcion)
      .then(async (horaExecpcion) => {
        if (
          (attrs.hora_inicio &&
            !attrs.hora_fin &&
            attrs.hora_inicio > horaExecpcion.hora_fin) ||
          (!attrs.hora_inicio &&
            attrs.hora_fin &&
            horaExecpcion.hora_inicio > attrs.hora_fin) ||
          (attrs.hora_inicio &&
            attrs.hora_fin &&
            attrs.hora_inicio > attrs.hora_fin)
        )
          throw new ConflictException(
            'La hora inicio no puede ser más grande que la hora fin.',
          );
        await this.validarHoras(
          horaExecpcion.institucionDia,
          attrs.hora_inicio,
          attrs.hora_fin,
        );
        Object.assign(horaExecpcion, attrs);
        return this.repository.save(horaExecpcion);
      })
      .then((_) => ({
        message:
          'Se guardo correctamente la información de esta hora excepción',
      }));
  }

  validarHoras(
    institucionDia: InstitucionDia,
    hora_inicio?: string,
    hora_fin?: string,
  ) {
    const busqueda: object[] = [
      {
        institucionDia,
        hora_inicio,
        hora_fin,
      },
    ];

    if (hora_inicio)
      busqueda.push({
        institucionDia,
        hora_inicio: LessThanOrEqual(hora_inicio),
        hora_fin: MoreThanOrEqual(hora_inicio),
      });
    if (hora_fin)
      busqueda.push({
        institucionDia,
        hora_inicio: LessThanOrEqual(hora_fin),
        hora_fin: MoreThanOrEqual(hora_fin),
      });
    return this.repository
      .findOne({
        where: busqueda,
      })
      .then((existeHoraExcepcion) => {
        if (existeHoraExcepcion) {
          if (
            existeHoraExcepcion.hora_fin === hora_fin &&
            existeHoraExcepcion.hora_inicio === hora_inicio
          )
            throw new ConflictException('Ya existe esta hora excepción.');
          throw new ConflictException(
            'Las horas escogídas se sobrelapan con otra hora excepción del mismo día.',
          );
        }
      });
  }
}
