import * as moment from 'moment';
import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOperator, Like, Repository } from 'typeorm';
import { Carrera } from '../institucion-carrera/entity/carrera.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Prestamo } from './entity/prestamo.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { CarritoService } from '../carrito/carrito.service';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionService } from '../institucion/institucion.service';
import { ModuloService } from '../modulo/modulo.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { InstitucionTipoEntradaService } from '../institucion-tipo-entrada/institucion-tipo-entrada.service';
import { OperadorService } from '../operador/operador.service';
import { StatusService } from '../status/status.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { UsuarioService } from '../usuario/usuario.service';

@Injectable()
export class PrestamoService {
  constructor(
    @InjectRepository(Prestamo) private repository: Repository<Prestamo>,
    private carritoService: CarritoService,
    private equipoService: EquipoService,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private institucionProgramaService: InstitucionProgramaService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
    private moduloService: ModuloService,
    private operadorService: OperadorService,
    private statusService: StatusService,
    private tipoUsuarioService: TipoUsuarioService,
    private usuarioService: UsuarioService,
  ) {}

  async create(
    id_usuario: number,
    id_modulo: number,
    id_tipo_carrito: number,
    id_programa?: number,
    id_tipo_entrada?: number,
  ) {
    const ahora = moment();
    const usuario = await this.usuarioService.findById(id_usuario, true, true);
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findTipoCarritoById(
        id_tipo_carrito,
      );
    const programa = id_programa
      ? await this.institucionProgramaService.findProgramaById(id_programa)
      : null;
    const tipoEntrada = id_tipo_entrada
      ? await this.institucionTipoEntradaService.findTipoEntradaById(
          id_tipo_entrada,
        )
      : null;

    console.log(modulo);

    return this.repository
      .findOne({ activo: true, usuario })
      .then((prestamo) => {
        if (prestamo)
          throw new ConflictException(
            'Este usuario ya tiene un prestamo activo.',
          );
        return this.equipoService.reseteo(
          modulo,
          tipoCarrito,
          programa,
          tipoEntrada,
        );
      })
      .then((_) =>
        this.equipoService
          .findEquipo(modulo, tipoCarrito, programa, tipoEntrada)
          .getOne(),
      )
      .then(async (equipo) => {
        if (!equipo)
          throw new ConflictException(
            'No hay un equipo de cómputo que cumpla con las caracteríasticas solicitasdas o ya no hay equipos disponibles en este momento. Intenta más tarde o cambia las caracteríasticas.',
          );
        equipo.status = await this.statusService.findById(2);
        return this.equipoService.update(equipo);
      })
      .then(({ equipo }) =>
        this.repository.save(
          this.repository.create({
            equipo,
            usuario,
            fecha_inicio: ahora.format(),
            hora_max_recoger: ahora
              .add(modulo.institucion.tiempo_recoger, 'm')
              .format(),
          }),
        ),
      );
  }

  async findAll(filtros: {
    id_modulo: string;
    pagina: string;
    activo?: string | boolean;
    carrito?: string;
    equipo?: string;
    fechaFin?: string;
    fechaInicio?: string;
    id_carrera?: string;
    id_operador_entrega?: string;
    id_operador_regreso?: string;
    id_tipo_carrito?: string;
    id_tipo_usuario?: string;
    id_prestamo?: string;
    usuario?: string;
  }) {
    const join: {
      e: string;
      u: string;
      c: string;
      m: string;
      tc: string;
      oe?: string;
      or?: string;
    } = {
      e: 'p.equipo',
      u: 'p.usuario',
      c: 'e.carrito',
      m: 'c.modulo',
      tc: 'c.tipoCarrito',
    };
    const busqueda: {
      activo?: boolean;
      fechaInicio?: Date;
      fechaFin?: Date;
      id_prestamo?: number;
      equipo: {
        equipo?: FindOperator<string>;
        carrito: {
          carrito?: FindOperator<string>;
          modulo?: Modulo;
          tipoCarrito?: TipoCarrito;
        };
      };
      usuario: {
        usuario?: FindOperator<string>;
        tipoUsuario?: TipoUsuario;
        institucionCarrera: { carrera?: Carrera };
      };
      operadorEntrega?: Operador;
      operadorRegreso?: Operador;
    } = { equipo: { carrito: {} }, usuario: { institucionCarrera: {} } };
    const carrera = filtros.id_carrera
      ? await this.institucionCarreraService.findCarreraByIdCarrera(
          parseInt(filtros.id_carrera),
        )
      : null;
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(
          parseInt(filtros.id_tipo_usuario),
        )
      : null;
    const operadorEntrega = filtros.id_operador_entrega
      ? await this.operadorService.findById(
          parseInt(filtros.id_operador_entrega),
        )
      : null;
    const operadorRegreso = filtros.id_operador_regreso
      ? await this.operadorService.findById(
          parseInt(filtros.id_operador_regreso),
        )
      : null;

    console.log(filtros.activo);
    if (filtros.activo) {
      if (typeof filtros.activo === 'boolean') busqueda.activo = filtros.activo;
      else busqueda.activo = filtros.activo === 'true';
    } else {
      console.log('hola');
      join.oe = 'p.operadorEntrega';
      join.or = 'p.operadorRegreso';
    }
    if (filtros.carrito)
      busqueda.equipo.carrito.carrito = Like(`%${filtros.carrito}%`);
    if (filtros.equipo) busqueda.equipo.equipo = Like(`%${filtros.equipo}%`);
    if (filtros.equipo) busqueda.id_prestamo = parseInt(filtros.id_prestamo);
    if (filtros.usuario)
      busqueda.usuario.usuario = Like(`%${filtros.usuario}%`);
    if (carrera) busqueda.usuario.institucionCarrera.carrera = carrera;
    if (modulo) busqueda.equipo.carrito.modulo = modulo;
    if (operadorEntrega) busqueda.operadorEntrega = operadorEntrega;
    if (operadorRegreso) busqueda.operadorRegreso = operadorRegreso;
    if (tipoCarrito) busqueda.equipo.carrito.tipoCarrito = tipoCarrito;
    if (tipoUsuario) busqueda.usuario.tipoUsuario = tipoUsuario;
    return this.repository.findAndCount({
      join: {
        alias: 'p',
        innerJoinAndSelect: join,
      },
      where: busqueda,
      skip: (parseInt(filtros.pagina) - 1) * 25,
      take: 25,
    });
  }

  findById(id_prestamo: number) {
    return this.repository.findOne({ id_prestamo }).then((prestamo) => {
      if (!prestamo) throw new NotFoundException('No existe este préstamo.');
      return prestamo;
    });
  }
}
